@@ -455,6 +455,9 @@ draw_arrow(GtkStyle* style,
 {
 	sanitize_size(window, &width, &height);
 	
+	if (gtkQtDebug)
+		printf("Arrow (%d,%d,%d,%d) Widget: %s  Detail: %s\n", x, y, width, height,gtk_widget_get_name(widget),detail);
+	
 	if (DETAIL("hscrollbar") || DETAIL("vscrollbar"))
 		return;
 	if (DETAIL("spinbutton"))
@@ -468,9 +471,12 @@ draw_arrow(GtkStyle* style,
 	{
 		GdkPixbuf *gpix;
 		GtkWidget* parent;
-		gpix = gdk_pixbuf_get_from_drawable(NULL, gtk_widget_get_parent_window(widget),NULL, x, y, 0, 0,  width, height);
-		setFillPixmap(gpix);
-		g_object_unref(gpix);
+		if (gdk_window_is_viewable(gtk_widget_get_parent_window(widget)))
+		{
+			gpix = gdk_pixbuf_get_from_drawable(NULL, gtk_widget_get_parent_window(widget),NULL, x, y, 0, 0,  width, height);
+			setFillPixmap(gpix);
+			g_object_unref(gpix);
+		}
 		
 		parent = gtk_widget_get_parent(widget);
 		drawArrow(window,style, GTK_WIDGET_STATE(parent), arrow_type, x, y, width, height);
@@ -555,9 +561,12 @@ draw_arrow(GtkStyle* style,
 	else
 	{
 		GdkPixbuf *gpix;
-		gpix = gdk_pixbuf_get_from_drawable(NULL, gtk_widget_get_parent_window(widget),NULL, x, y, 0, 0,  width, height);
-		setFillPixmap(gpix);
-		g_object_unref(gpix);
+		if (gdk_window_is_viewable(gtk_widget_get_parent_window(widget)))
+		{
+			gpix = gdk_pixbuf_get_from_drawable(NULL, gtk_widget_get_parent_window(widget),NULL, x, y, 0, 0,  width, height);
+			setFillPixmap(gpix);
+			g_object_unref(gpix);
+		}
 		
 		drawArrow(window, style, state_type, arrow_type, x, y, width, height);
  		return;  
@@ -627,13 +636,21 @@ draw_box(GtkStyle * style,
 	}
 	if (DETAIL("menubar"))
 	{
-		drawMenubar(window,style,state_type,x,y,width,height);
+		if (openOfficeFix == 1)
+			 parent_class->draw_box (style, window, state_type, shadow_type, area, widget, detail, x, y, width, height);
+		else
+			drawMenubar(window,style,state_type,x,y,width,height);
 		return;
 	}
 	if (DETAIL("menu"))
 	{
-		if ((x >= 0) && (y >= 0)) /* Work around weirdness in firefox */
-			drawMenu(window,style,state_type,x,y,width,height);
+		if (openOfficeFix == 1)
+			 parent_class->draw_box (style, window, state_type, shadow_type, area, widget, detail, x, y, width, height);
+		else
+		{
+			if ((x >= 0) && (y >= 0)) /* Work around weirdness in firefox */
+				drawMenu(window,style,state_type,x,y,width,height);
+		}
 		return;
 	}
 	if (GTK_IS_PROGRESS(widget) && DETAIL("trough"))
@@ -646,19 +663,35 @@ draw_box(GtkStyle * style,
 	}
 	if (GTK_IS_PROGRESS(widget) && DETAIL("bar"))
 	{
+		if (area) gdk_gc_set_clip_rectangle(style->bg_gc[state_type], area);
+		
 		drawProgressChunk(window,style,state_type,x,y,width,height);
+		
+		if (area) gdk_gc_set_clip_rectangle(style->bg_gc[state_type], NULL);
 		return;
 	}
 	if (GTK_IS_SCALE(widget) && DETAIL("trough"))
 	{
-		GdkPixbuf *gpix;
 		GtkAdjustment* adj;
-		gpix = gdk_pixbuf_get_from_drawable(NULL, gtk_widget_get_parent_window(widget),NULL, x, y, 0, 0,  width, height);
-		setFillPixmap(gpix);
-		g_object_unref(gpix);
+		int inverted;
+		GValue *val = (GValue*)g_malloc( sizeof(GValue) );
+		if (gdk_window_is_viewable(gtk_widget_get_parent_window(widget)))
+		{
+			GdkPixbuf *gpix;
+			gpix = gdk_pixbuf_get_from_drawable(NULL, gtk_widget_get_parent_window(widget),NULL, x, y, 0, 0,  width, height);
+			setFillPixmap(gpix);
+			g_object_unref(gpix);
+		}
+		
+		memset( val, 0, sizeof(GValue) );
+		g_value_init( val, G_TYPE_BOOLEAN );
+		g_object_get_property(widget, "inverted", val);
+		inverted = g_value_get_boolean(val);
+		g_value_unset(val);
+		g_free(val);
 		
 		adj = gtk_range_get_adjustment((GtkRange *) widget);
-		drawSlider(window,style,state_type,adj,x,y,width,height, (GTK_RANGE(widget))->orientation);
+		drawSlider(window,style,state_type,adj,x,y,width,height, (GTK_RANGE(widget))->orientation, inverted);
 		return;
 	}
 	if (DETAIL("button"))
@@ -767,7 +800,10 @@ draw_box(GtkStyle * style,
 	}
 	if (DETAIL("toolbar"))
 	{
-		drawToolbar(window,style,state_type,x,y,width,height);
+		if (openOfficeFix == 1)
+			parent_class->draw_box (style, window, state_type, shadow_type, area, widget, detail, x, y, width, height);
+		else
+			drawToolbar(window,style,state_type,x,y,width,height);
 		return;
 	}
 	if (DETAIL("spinbutton_up"))
@@ -862,14 +898,20 @@ draw_check(GtkStyle * style,
            gint width,
            gint height)
 {
+	if (gtkQtDebug)
+		printf("Check (%d,%d,%d,%d) Widget: %s  Detail: %s\n", x, y, width, height,gtk_widget_get_name(widget),detail);
+	
 	if (GTK_IS_MENU_ITEM(widget))
 	{
 		if (shadow_type == GTK_SHADOW_IN)
 		{
-			GdkPixbuf *gpix;
-			gpix = gdk_pixbuf_get_from_drawable(NULL, gtk_widget_get_parent_window(widget), NULL, x, y, 0, 0, width, height);
-			setFillPixmap(gpix);
-			g_object_unref(gpix);
+			if (gdk_window_is_viewable(gtk_widget_get_parent_window(widget)))
+			{
+				GdkPixbuf *gpix;
+				gpix = gdk_pixbuf_get_from_drawable(NULL, gtk_widget_get_parent_window(widget), NULL, x, y, 0, 0, width, height);
+				setFillPixmap(gpix);
+				g_object_unref(gpix);
+			}
 		
 			drawMenuCheck(window,style,state_type,x,y,width,height);
 		}
@@ -893,10 +935,16 @@ draw_option(GtkStyle * style,
             gint width,
             gint height)
 {
-	GdkPixbuf *gpix;
-	gpix = gdk_pixbuf_get_from_drawable(NULL, gtk_widget_get_parent_window(widget),NULL, x, y, 0, 0,  width, height);
-	setFillPixmap(gpix);
-	g_object_unref(gpix);
+	if (gtkQtDebug)
+		printf("Option (%d,%d,%d,%d) Widget: %s  Detail: %s\n", x, y, width, height,gtk_widget_get_name(widget),detail);
+	
+	if (gdk_window_is_viewable(gtk_widget_get_parent_window(widget)))
+	{
+		GdkPixbuf *gpix;
+		gpix = gdk_pixbuf_get_from_drawable(NULL, gtk_widget_get_parent_window(widget),NULL, x, y, 0, 0,  width, height);
+		setFillPixmap(gpix);
+		g_object_unref(gpix);
+	}
 
 	if (GTK_IS_MENU_ITEM(widget))
 	{
@@ -921,6 +969,9 @@ draw_tab(GtkStyle * style,
          gint width,
          gint height)
 {
+	if (gtkQtDebug)
+		printf("Tab (%d,%d,%d,%d) Widget: %s  Detail: %s\n", x, y, width, height,gtk_widget_get_name(widget),detail);
+	
 	gtk_paint_box(style, window, state_type, shadow_type, area, widget, detail, x, y, width, height);
 }
 
@@ -1090,6 +1141,12 @@ draw_box_gap(GtkStyle* style,
              gint gap_width)
 {
 	sanitize_size (window, &width, &height);
+	
+	if (width<0 || height<0) return;  /* Eclipse really can be this stupid! */
+	
+	if (gtkQtDebug)
+		printf("Box_gap (%d,%d,%d,%d) Widget: %s  Detail: %s\n", x, y, width, height,gtk_widget_get_name(widget),detail);
+	
 	if (DETAIL("notebook"))
 		drawTabFrame(window,style,state_type,x,y-2,width,height+2, gtk_notebook_get_tab_pos((GtkNotebook *)widget));
 }
@@ -1114,6 +1171,9 @@ draw_extension(GtkStyle * style,
 	
 	sanitize_size (window, &width, &height);
 	
+	if (gtkQtDebug)
+		printf("Extension (%d,%d,%d,%d) Widget: %s  Detail: %s\n", x, y, width, height,gtk_widget_get_name(widget),detail);
+	
 	gtk_paint_box(style, window, state_type, shadow_type, area, widget, detail,
 			x, y, width, height);
 }
@@ -1148,6 +1208,9 @@ draw_slider(GtkStyle * style,
             gint height,
             GtkOrientation orientation)
 {
+	if (gtkQtDebug)
+		printf("Slider (%d,%d,%d,%d) Widget: %s  Detail: %s\n", x, y, width, height,gtk_widget_get_name(widget),detail);
+	
 	if (DETAIL("slider"))
 	{
 		GtkAdjustment* adj = gtk_range_get_adjustment(GTK_RANGE(widget));
@@ -1227,6 +1290,27 @@ void  draw_layout  (GtkStyle *style,
 				PangoAttribute *textcolorattr;
 				text_length = g_utf8_strlen (text, -1);
 				text_bytelen = strlen (text);
+
+				/* Try to get the attribute list of the layout */
+				PangoAttrList* attrlist = pango_layout_get_attributes(layout);
+				if(attrlist) {
+					/* Now iterate over the attribute list */
+					PangoAttrIterator* it = pango_attr_list_get_iterator(attrlist);
+					if(it) {
+						/* Try to get the first foreground color attribute
+						   Note that there can be more than one foreground 
+						   color, we use only the first */
+						PangoAttrColor* fg = (PangoAttrColor*)pango_attr_iterator_get(it, PANGO_ATTR_FOREGROUND);
+						if(fg) {
+							/* Use this color to render the text, instead of
+							   the default text color */
+							color.red	= fg->color.red;
+							color.green	= fg->color.green;
+							color.blue	= fg->color.blue;
+						}
+						pango_attr_iterator_destroy(it);
+					}
+				}
 				
 				textcolorattr = pango_attr_foreground_new(color.red, color.green, color.blue);
 				textcolorattr->start_index = 0;
@@ -1777,8 +1861,10 @@ set_background (GtkStyle *style, GdkWindow *window, GtkStateType state_type)
         {
           pixmap = pix_test;
           parent_relative = FALSE;
+	  gdk_drawable_set_colormap(pixmap, style->colormap);
         }
       
+      if (pixmap && !gdk_drawable_get_colormap (pixmap)) gdk_drawable_set_colormap (pixmap, gdk_drawable_get_colormap (window));
       gdk_window_set_back_pixmap (window, pixmap, parent_relative);
     }
   else
